# NextCloud WebDAV Client

A Drupal module that provides integration with NextCloud via WebDAV protocol, allowing you to create folders and upload files to your NextCloud instance.

## Features

- **Folder Creation**: Create single folders or nested folder structures
- **File Upload**: Upload files from local filesystem or Drupal file entities
- **Content Upload**: Upload text content directly as files
- **Directory Listing**: List files and folders in NextCloud directories
- **Batch Operations**: Upload multiple files at once
- **Configuration Management**: Easy-to-use admin interface for connection settings
- **Connection Testing**: Built-in connection test functionality
- **Comprehensive Logging**: Detailed logging for debugging and monitoring

## Requirements

- Drupal 10 or 11
- NextCloud server with WebDAV enabled
- Valid NextCloud user credentials

## Installation

1. Install the module via Composer:
   ```bash
   composer require drupal/nextcloud_webdav_client
   ```
2. Enable the module:
   ```bash
   drush en nextcloud_webdav_client
   ```
3. Configure the connection settings at `/admin/config/services/nextcloud-webdav`

## Configuration

Navigate to **Administration > Configuration > Web Services > NextCloud WebDAV** to configure:

### Connection Settings
- **Server URL**: Your NextCloud server URL (e.g., `https://cloud.example.com`)
- **Username**: Your NextCloud username
- **Password**: Your NextCloud password or app password (recommended)
- **Base Path**: WebDAV base path (usually `/remote.php/dav/files/`)

### Advanced Settings
- **Timeout**: Request timeout in seconds (default: 30)
- **Verify SSL**: Whether to verify SSL certificates (disable only for development)

### Security Recommendations
- Use NextCloud app passwords instead of your main password
- Enable SSL/TLS on your NextCloud server
- Keep the "Verify SSL" option enabled in production

## Usage

### Programmatic Usage

#### Using the Manager Service

```php
// Get the manager service
$webdav_manager = \Drupal::service('nextcloud_webdav_client.manager');

// Create a folder
$webdav_manager->createFolder('my-folder/subfolder');

// Upload a Drupal file entity
$file = File::load($file_id);
$webdav_manager->uploadDrupalFile($file, 'uploads/myfile.pdf');

// Upload content as a file
$content = "Hello, NextCloud!";
$webdav_manager->uploadContent($content, 'messages/hello.txt');

// Upload a local file
$webdav_manager->uploadFile('/path/to/local/file.pdf', 'documents/file.pdf');

// List directory contents
$items = $webdav_manager->listDirectory('my-folder');
```

#### Using the Client Service Directly

```php
// Get the client service
$webdav_client = \Drupal::service('nextcloud_webdav_client.client');

// Test connection
if ($webdav_client->testConnection()) {
  // Connection successful
}

// Create folder
$webdav_client->createFolder('test-folder');

// Upload file
$webdav_client->uploadFile('/local/path/file.txt', 'remote/path/file.txt');
```

### Batch Operations

```php
$webdav_manager = \Drupal::service('nextcloud_webdav_client.manager');

// Upload multiple files
$files = [
  ['local_path' => '/path/to/file1.pdf', 'remote_path' => 'docs/file1.pdf'],
  ['local_path' => '/path/to/file2.pdf', 'remote_path' => 'docs/file2.pdf'],
];

$results = $webdav_manager->uploadMultipleFiles($files);
// $results contains 'success' and 'failed' arrays
```

### Testing Interface

Visit `/admin/config/services/nextcloud-webdav/test` to access the testing interface where you can:
- Create folders
- Upload files via web interface
- Upload text content
- List directory contents

## API Reference

### NextCloudWebDavManager

The main service for high-level operations.

#### Methods

- `isConfigured()`: Check if module is configured
- `testConnection()`: Test connection to NextCloud
- `createFolder(string $path)`: Create folder(s)
- `uploadDrupalFile(FileInterface $file, string $remote_path, bool $create_folders)`: Upload Drupal file
- `uploadFile(string $local_path, string $remote_path, bool $create_folders)`: Upload local file
- `uploadContent(string $content, string $remote_path, bool $create_folders)`: Upload content as file
- `listDirectory(string $path)`: List directory contents
- `uploadMultipleFiles(array $files, bool $create_folders)`: Batch upload files

### NextCloudWebDavClient

The low-level WebDAV client service.

#### Methods

- `testConnection()`: Test WebDAV connection
- `createFolder(string $path)`: Create single folder
- `uploadFile(string $local_path, string $remote_path)`: Upload file
- `uploadContent(string $content, string $remote_path)`: Upload content
- `listDirectory(string $path)`: List directory
- `isConfigured()`: Check configuration

## Permissions

The module defines two permissions:

- **Administer NextCloud WebDAV**: Configure connection settings
- **Use NextCloud WebDAV**: Use WebDAV functionality and test interface

## Hooks and Events

### Custom Module Integration

You can integrate this module into your custom modules:

```php
/**
 * Implements hook_file_insert().
 */
function mymodule_file_insert(FileInterface $file) {
  $webdav_manager = \Drupal::service('nextcloud_webdav_client.manager');
  
  if ($webdav_manager->isConfigured()) {
    // Auto-upload files to NextCloud
    $remote_path = 'drupal-uploads/' . $file->getFilename();
    $webdav_manager->uploadDrupalFile($file, $remote_path);
  }
}
```

## Troubleshooting

### Common Issues

1. **Connection Failed**
   - Verify server URL is correct and accessible
   - Check username and password
   - Ensure NextCloud WebDAV is enabled
   - Check SSL certificate issues

2. **Upload Failures**
   - Verify file permissions on local files
   - Check NextCloud storage quota
   - Ensure target directory exists or enable folder creation

3. **SSL Certificate Errors**
   - For development: Disable SSL verification (not recommended for production)
   - For production: Ensure valid SSL certificate

### Debugging

Enable logging and check the logs at **Reports > Recent log messages** for detailed error information.

## Security Considerations

- Always use HTTPS for NextCloud connections
- Use NextCloud app passwords instead of main passwords
- Regularly rotate credentials
- Monitor file upload permissions and quotas
- Keep NextCloud server updated

## Contributing

This module follows Drupal coding standards and best practices. When contributing:

1. Follow Drupal coding standards
2. Add appropriate documentation
3. Include error handling and logging
4. Write tests for new functionality

## License

This module is licensed under the GPL v2 or later. 