<?php

namespace Drupal\nextcloud_webdav_client\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\nextcloud_webdav_client\Service\NextCloudWebDavClient;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure NextCloud WebDAV settings.
 */
class NextCloudWebDavSettingsForm extends ConfigFormBase {

  /**
   * The NextCloud WebDAV client service.
   *
   * @var \Drupal\nextcloud_webdav_client\Service\NextCloudWebDavClient
   */
  protected $webdavClient;

  /**
   * Constructs a NextCloudWebDavSettingsForm object.
   *
   * @param \Drupal\nextcloud_webdav_client\Service\NextCloudWebDavClient $webdav_client
   *   The NextCloud WebDAV client service.
   */
  public function __construct(NextCloudWebDavClient $webdav_client) {
    $this->webdavClient = $webdav_client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('nextcloud_webdav_client.client')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['nextcloud_webdav_client.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'nextcloud_webdav_client_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('nextcloud_webdav_client.settings');

    $form['connection'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Connection Settings'),
      '#collapsible' => FALSE,
    ];

    $form['connection']['server_url'] = [
      '#type' => 'url',
      '#title' => $this->t('Server URL'),
      '#description' => $this->t('The base URL of your NextCloud server (e.g., https://cloud.example.com)'),
      '#default_value' => $config->get('server_url'),
      '#required' => TRUE,
    ];

    $form['connection']['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#description' => $this->t('Your NextCloud username'),
      '#default_value' => $config->get('username'),
      '#required' => TRUE,
    ];

    // Check if password already exists
    $existing_password = $config->get('password');
    $password_description = $this->t('Your NextCloud password or app password (recommended)');
    
    if (!empty($existing_password)) {
      $password_description = $this->t('Your NextCloud password or app password (recommended). Leave empty to keep the current password.');
      $form['connection']['password_status'] = [
        '#type' => 'markup',
        '#markup' => '<div class="messages messages--status">' . 
          $this->t('✓ Password is currently set. Leave the password field empty to keep the existing password.') . 
          '</div>',
      ];
    }

    $form['connection']['password'] = [
      '#type' => 'password',
      '#title' => $this->t('Password'),
      '#description' => $password_description,
      '#required' => empty($existing_password), // Only required if no password exists
    ];

    $form['connection']['base_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base Path'),
      '#description' => $this->t('The WebDAV base path (usually /remote.php/dav/files/)'),
      '#default_value' => $config->get('base_path') ?: '/remote.php/dav/files/',
      '#required' => TRUE,
    ];

    $form['advanced'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Advanced Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    ];

    $form['advanced']['timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Timeout'),
      '#description' => $this->t('Request timeout in seconds'),
      '#default_value' => $config->get('timeout') ?: 30,
      '#min' => 1,
      '#max' => 300,
    ];

    $form['advanced']['verify_ssl'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Verify SSL certificates'),
      '#description' => $this->t('Uncheck this only for development or self-signed certificates'),
      '#default_value' => $config->get('verify_ssl') !== FALSE,
    ];

    // Add security information
    $form['security_info'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Security Information'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    ];

    $form['security_info']['info'] = [
      '#type' => 'markup',
      '#markup' => '<div class="description">' .
        '<h4>' . $this->t('Password Storage') . '</h4>' .
        '<p>' . $this->t('Passwords are stored in Drupal\'s configuration system. While this is encrypted in transit and at rest on properly configured servers, consider these security best practices:') . '</p>' .
        '<ul>' .
        '<li>' . $this->t('<strong>Use NextCloud App Passwords:</strong> Generate a dedicated app password in NextCloud instead of using your main password.') . '</li>' .
        '<li>' . $this->t('<strong>Limit Permissions:</strong> Create a dedicated NextCloud user with minimal required permissions.') . '</li>' .
        '<li>' . $this->t('<strong>Regular Rotation:</strong> Rotate passwords regularly and revoke unused app passwords.') . '</li>' .
        '<li>' . $this->t('<strong>HTTPS Only:</strong> Always use HTTPS for your NextCloud server.') . '</li>' .
        '</ul>' .
        '<p><strong>' . $this->t('How to create NextCloud App Password:') . '</strong></p>' .
        '<ol>' .
        '<li>' . $this->t('Log into your NextCloud web interface') . '</li>' .
        '<li>' . $this->t('Go to Settings > Personal > Security') . '</li>' .
        '<li>' . $this->t('Scroll to "App passwords" section') . '</li>' .
        '<li>' . $this->t('Enter a name (e.g., "Drupal WebDAV") and click "Create new app password"') . '</li>' .
        '<li>' . $this->t('Copy the generated password and use it here') . '</li>' .
        '</ol>' .
        '</div>',
    ];

    $form['test'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Connection Test'),
      '#collapsible' => FALSE,
    ];

    $form['test']['test_connection'] = [
      '#type' => 'button',
      '#value' => $this->t('Test Connection'),
      '#ajax' => [
        'callback' => '::testConnectionCallback',
        'wrapper' => 'test-result',
        'method' => 'replace',
        'effect' => 'fade',
      ],
    ];

    $form['test']['test_result'] = [
      '#type' => 'markup',
      '#markup' => '',
      '#prefix' => '<div id="test-result">',
      '#suffix' => '</div>',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Ajax callback for testing the connection.
   */
  public function testConnectionCallback(array &$form, FormStateInterface $form_state) {
    // Get form values
    $test_password = $form_state->getValue('password');
    $existing_password = $this->config('nextcloud_webdav_client.settings')->get('password');
    
    // Use existing password if no new password provided
    $password_to_test = !empty($test_password) ? $test_password : $existing_password;
    
    // Temporarily update config with form values for testing.
    $temp_config = [
      'server_url' => $form_state->getValue('server_url'),
      'username' => $form_state->getValue('username'),
      'password' => $password_to_test,
      'base_path' => $form_state->getValue('base_path'),
      'timeout' => $form_state->getValue('timeout'),
      'verify_ssl' => $form_state->getValue('verify_ssl'),
    ];

    // Save current config.
    $current_config = $this->config('nextcloud_webdav_client.settings');
    $original_values = [];
    foreach ($temp_config as $key => $value) {
      $original_values[$key] = $current_config->get($key);
    }

    // Set temporary values.
    $config = $this->configFactory->getEditable('nextcloud_webdav_client.settings');
    foreach ($temp_config as $key => $value) {
      $config->set($key, $value);
    }
    $config->save();

    // Test connection.
    $success = $this->webdavClient->testConnection();

    // Restore original values.
    foreach ($original_values as $key => $value) {
      $config->set($key, $value);
    }
    $config->save();

    // Prepare result message.
    if ($success) {
      $message = [
        '#type' => 'markup',
        '#markup' => '<div class="messages messages--status">' . $this->t('Connection successful!') . '</div>',
      ];
    }
    else {
      $message = [
        '#type' => 'markup',
        '#markup' => '<div class="messages messages--error">' . $this->t('Connection failed. Please check your settings.') . '</div>',
      ];
    }

    $form['test']['test_result'] = $message + [
      '#prefix' => '<div id="test-result">',
      '#suffix' => '</div>',
    ];

    return $form['test']['test_result'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('nextcloud_webdav_client.settings');
    
    // Handle password: only update if a new password was provided
    $new_password = $form_state->getValue('password');
    $existing_password = $config->get('password');
    
    $password_to_save = !empty($new_password) ? $new_password : $existing_password;

    $config
      ->set('server_url', $form_state->getValue('server_url'))
      ->set('username', $form_state->getValue('username'))
      ->set('password', $password_to_save)
      ->set('base_path', $form_state->getValue('base_path'))
      ->set('timeout', $form_state->getValue('timeout'))
      ->set('verify_ssl', $form_state->getValue('verify_ssl'))
      ->save();

    // Add a message about password handling
    if (empty($new_password) && !empty($existing_password)) {
      $this->messenger()->addStatus($this->t('Settings saved. The existing password was kept unchanged.'));
    }
    else {
      $this->messenger()->addStatus($this->t('Settings saved successfully.'));
    }
  }

} 