<?php

namespace Drupal\nextcloud_webdav_client\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\nextcloud_webdav_client\NextCloudWebDavManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Test form for NextCloud WebDAV operations.
 */
class NextCloudWebDavTestForm extends FormBase {

  /**
   * The NextCloud WebDAV manager service.
   *
   * @var \Drupal\nextcloud_webdav_client\NextCloudWebDavManager
   */
  protected $webdavManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a NextCloudWebDavTestForm object.
   *
   * @param \Drupal\nextcloud_webdav_client\NextCloudWebDavManager $webdav_manager
   *   The NextCloud WebDAV manager service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(NextCloudWebDavManager $webdav_manager, MessengerInterface $messenger) {
    $this->webdavManager = $webdav_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('nextcloud_webdav_client.manager'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'nextcloud_webdav_client_test';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    if (!$this->webdavManager->isConfigured()) {
      $form['not_configured'] = [
        '#type' => 'markup',
        '#markup' => '<div class="messages messages--warning">' . 
          $this->t('NextCloud WebDAV is not configured. Please <a href="@url">configure it first</a>.', [
            '@url' => '/admin/config/services/nextcloud-webdav',
          ]) . '</div>',
      ];
      return $form;
    }

    $form['folder_operations'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Folder Operations'),
      '#collapsible' => FALSE,
    ];

    $form['folder_operations']['folder_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Folder Path'),
      '#description' => $this->t('Enter the folder path to create (e.g., "test-folder" or "parent/child")'),
      '#default_value' => 'drupal-test-folder',
    ];

    $form['folder_operations']['create_folder'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create Folder'),
      '#submit' => ['::createFolderSubmit'],
    ];

    $form['file_operations'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('File Operations'),
      '#collapsible' => FALSE,
    ];

    $form['file_operations']['file_upload'] = [
      '#type' => 'file',
      '#title' => $this->t('Upload File'),
      '#description' => $this->t('Select a file to upload to NextCloud'),
    ];

    $form['file_operations']['remote_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Remote Path'),
      '#description' => $this->t('Enter the remote path (optional, defaults to filename)'),
      '#placeholder' => 'folder/filename.ext',
    ];

    $form['file_operations']['upload_file'] = [
      '#type' => 'submit',
      '#value' => $this->t('Upload File'),
      '#submit' => ['::uploadFileSubmit'],
    ];

    $form['content_operations'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Content Operations'),
      '#collapsible' => FALSE,
    ];

    $form['content_operations']['content'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Content'),
      '#description' => $this->t('Enter content to upload as a file'),
      '#default_value' => 'Hello from Drupal!\nThis is a test file created by the NextCloud WebDAV module.',
    ];

    $form['content_operations']['content_filename'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Filename'),
      '#description' => $this->t('Enter the filename for the content'),
      '#default_value' => 'drupal-test.txt',
    ];

    $form['content_operations']['upload_content'] = [
      '#type' => 'submit',
      '#value' => $this->t('Upload Content'),
      '#submit' => ['::uploadContentSubmit'],
    ];

    $form['list_operations'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('List Operations'),
      '#collapsible' => FALSE,
    ];

    $form['list_operations']['list_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Directory Path'),
      '#description' => $this->t('Enter the directory path to list (leave empty for root)'),
      '#placeholder' => 'folder-name',
    ];

    $form['list_operations']['list_directory'] = [
      '#type' => 'submit',
      '#value' => $this->t('List Directory'),
      '#submit' => ['::listDirectorySubmit'],
    ];

    $form['list_operations']['debug_directory'] = [
      '#type' => 'submit',
      '#value' => $this->t('Debug Directory Listing'),
      '#submit' => ['::debugDirectorySubmit'],
    ];

    return $form;
  }

  /**
   * Submit handler for creating folders.
   */
  public function createFolderSubmit(array &$form, FormStateInterface $form_state) {
    $folder_path = $form_state->getValue('folder_path');
    
    if (empty($folder_path)) {
      $this->messenger->addError($this->t('Please enter a folder path.'));
      return;
    }

    if ($this->webdavManager->createFolder($folder_path)) {
      $this->messenger->addStatus($this->t('Folder "@path" created successfully.', ['@path' => $folder_path]));
    }
    else {
      $this->messenger->addError($this->t('Failed to create folder "@path".', ['@path' => $folder_path]));
    }
  }

  /**
   * Submit handler for uploading files.
   */
  public function uploadFileSubmit(array &$form, FormStateInterface $form_state) {
    $files = $this->getRequest()->files->get('files');
    
    if (empty($files['file_upload']) || $files['file_upload']->getError() !== UPLOAD_ERR_OK) {
      $this->messenger->addError($this->t('Please select a file to upload.'));
      return;
    }

    $uploaded_file = $files['file_upload'];
    $local_path = $uploaded_file->getPathname();
    $remote_path = $form_state->getValue('remote_path') ?: $uploaded_file->getClientOriginalName();

    if ($this->webdavManager->uploadFile($local_path, $remote_path)) {
      $this->messenger->addStatus($this->t('File "@filename" uploaded successfully to "@path".', [
        '@filename' => $uploaded_file->getClientOriginalName(),
        '@path' => $remote_path,
      ]));
    }
    else {
      $this->messenger->addError($this->t('Failed to upload file "@filename".', [
        '@filename' => $uploaded_file->getClientOriginalName(),
      ]));
    }
  }

  /**
   * Submit handler for uploading content.
   */
  public function uploadContentSubmit(array &$form, FormStateInterface $form_state) {
    $content = $form_state->getValue('content');
    $filename = $form_state->getValue('content_filename');

    if (empty($content)) {
      $this->messenger->addError($this->t('Please enter content to upload.'));
      return;
    }

    if (empty($filename)) {
      $this->messenger->addError($this->t('Please enter a filename.'));
      return;
    }

    if ($this->webdavManager->uploadContent($content, $filename)) {
      $this->messenger->addStatus($this->t('Content uploaded successfully as "@filename".', ['@filename' => $filename]));
    }
    else {
      $this->messenger->addError($this->t('Failed to upload content as "@filename".', ['@filename' => $filename]));
    }
  }

  /**
   * Submit handler for listing directories.
   */
  public function listDirectorySubmit(array &$form, FormStateInterface $form_state) {
    $path = $form_state->getValue('list_path');
    $items = $this->webdavManager->listDirectory($path);

    if ($items === FALSE) {
      $this->messenger->addError($this->t('Failed to list directory "@path". Check the logs for more details.', ['@path' => $path ?: '/']));
      return;
    }

    if (empty($items)) {
      $this->messenger->addStatus($this->t('Directory "@path" is empty or contains no accessible items.', ['@path' => $path ?: '/']));
      return;
    }

    $list = [];
    foreach ($items as $item) {
      $type_icon = $item['type'] === 'directory' ? '📁' : '📄';
      $size_info = !empty($item['size']) && $item['type'] === 'file' ? ' (' . $this->formatBytes($item['size']) . ')' : '';
      $list[] = $type_icon . ' ' . $item['name'] . $size_info;
    }

    $this->messenger->addStatus($this->t('Directory "@path" contains @count items: @items', [
      '@path' => $path ?: '/',
      '@count' => count($items),
      '@items' => implode(', ', $list),
    ]));
  }

  /**
   * Submit handler for debugging directory listing.
   */
  public function debugDirectorySubmit(array &$form, FormStateInterface $form_state) {
    $path = $form_state->getValue('list_path');
    $client = \Drupal::service('nextcloud_webdav_client.client');
    $debug_info = $client->debugListDirectory($path);

    $debug_output = '<h4>Debug Information:</h4>';
    $debug_output .= '<p><strong>URL:</strong> ' . $debug_info['url'] . '</p>';
    $debug_output .= '<p><strong>Status Code:</strong> ' . $debug_info['status_code'] . '</p>';
    
    if (!empty($debug_info['error'])) {
      $debug_output .= '<p><strong>Error:</strong> ' . $debug_info['error'] . '</p>';
    }
    
    if (!empty($debug_info['response_body'])) {
      $debug_output .= '<p><strong>Response Body:</strong></p>';
      $debug_output .= '<pre style="background: #f5f5f5; padding: 10px; overflow: auto; max-height: 300px;">';
      $debug_output .= htmlspecialchars($debug_info['response_body']);
      $debug_output .= '</pre>';
    }

    $this->messenger->addStatus($this->t('Debug information: @debug', [
      '@debug' => '',
    ]));
    
    // Use drupal_set_message equivalent for raw HTML
    \Drupal::messenger()->addMessage(\Drupal\Core\Render\Markup::create($debug_output));
  }

  /**
   * Helper function to format bytes into human readable format.
   *
   * @param int $bytes
   *   The number of bytes.
   *
   * @return string
   *   Formatted string.
   */
  protected function formatBytes($bytes) {
    if ($bytes == 0) {
      return '0 B';
    }
    
    $units = ['B', 'KB', 'MB', 'GB', 'TB'];
    $factor = floor(log($bytes, 1024));
    
    return sprintf('%.1f %s', $bytes / pow(1024, $factor), $units[$factor]);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Default submit handler - not used as we have specific submit handlers.
  }

} 